/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.commands {
import com.adobe.cairngorm.business.ServiceLocator;
import com.adobe.cairngorm.commands.ICommand;
import com.adobe.cairngorm.control.CairngormEvent;

import mx.collections.ArrayCollection;
import mx.managers.CursorManager;
import mx.rpc.AsyncToken;
import mx.rpc.Responder;
import mx.rpc.events.FaultEvent;
import mx.rpc.events.ResultEvent;
import mx.rpc.remoting.RemoteObject;

import org.ow2.orchestra.designer.bpmn.model.ProcessModel;
import org.ow2.orchestra.designer.events.FileNavigatorEvent;
import org.ow2.orchestra.designer.model.DesignerModelLocator;
import org.ow2.orchestra.designer.model.FileNavigatorModel;
import org.ow2.orchestra.designer.model.ProcessEditorModel;
import org.ow2.orchestra.designer.utils.ErrorManager;
import org.ow2.orchestra.designer.vo.FileNavigatorProcess;
import org.ow2.orchestra.designer.vo.FileNavigatorUserAccessPermissions;

/**
 * To open a file
 * @author lizq
 *
 */
public class ProcessOpenCMD implements ICommand {

  private var process:FileNavigatorProcess;
  private var processEditorModel:ProcessEditorModel = new ProcessEditorModel();

  public function ProcessOpenCMD() {
    super();
  }

  /**
   * Open a new process file or select the process if already opened.
   * @param event filePath
   */
  public function execute(event:CairngormEvent):void {
    trace("Command.ProcessOpenCMD");
    // save file path to store into model
    var fileNavigatorEvent:FileNavigatorEvent = event as FileNavigatorEvent;
    process = fileNavigatorEvent.item as FileNavigatorProcess;
    // check if processModel is already opened
    var processEditorModel:ProcessEditorModel =
        DesignerModelLocator.getInstance().getOpenProcessByFilePath(process);
    if (processEditorModel != null) {
      // select process editor model if exists
      DesignerModelLocator.getInstance().selectedProcessEditor = processEditorModel;
    }
    else {
      // read process model from server file
      // call open file on remote server
      var remote:RemoteObject = ServiceLocator.getInstance().getRemoteObject("fileNavigator");

      CursorManager.setBusyCursor();
      var fileNavigatorViewModel:FileNavigatorModel = DesignerModelLocator.getInstance().fileNavigatorModel;
      // disable double click into FileNavigator to prevent multiple opening of the same process
      fileNavigatorViewModel.isEnabledDoubleClick = false;

      var token:AsyncToken = remote.openFile(process);
      // add fonctions to call on result
      token.addResponder(new Responder(openFileResult, fault));
    }

  }


  private function openFileResult(event:ResultEvent):void {
    // create process model
    processEditorModel.fileNavigatorProcess = process;
    processEditorModel.processModel.name = process.name;

    processEditorModel.processModel = event.result as ProcessModel;

    if (DesignerModelLocator.getInstance().userName != (process as FileNavigatorProcess).project.owner.name) {
      var remote:RemoteObject = ServiceLocator.getInstance().getRemoteObject("fileNavigator");
      var token:AsyncToken = remote.getAccessProperties(process);
      // add fonctions to call on result
      token.addResponder(new Responder(getOwnAdminProcessResult, fault));
    } else {
      var remote:RemoteObject = ServiceLocator.getInstance().getRemoteObject("fileNavigator");
      var token:AsyncToken = remote.whoTakeToken(process);
      // add fonctions to call on result
      token.addResponder(new Responder(checkWhoGetTokenResult, fault));
    }
  }

  private function fault(event:FaultEvent):void {
    CursorManager.removeBusyCursor();
    // allow now opening process with double click on FileNavigator
    DesignerModelLocator.getInstance().fileNavigatorModel.isEnabledDoubleClick = true;

    ErrorManager.getInstance().updateTitle("Error : ");
    ErrorManager.getInstance().showError("Remote invoke error: " + event.message);
  }

  private function getOwnAdminProcessResult(event:ResultEvent):void {
    var access:ArrayCollection = event.result as ArrayCollection;

    for each(var userAccessPermissions:FileNavigatorUserAccessPermissions in access) {
      if (userAccessPermissions.user == DesignerModelLocator.getInstance().userName) {
        if (!userAccessPermissions.writing) {
          processEditorModel.readWrite = false;
          ErrorManager.getInstance().updateTitle("Warning : read-only process");
          ErrorManager.getInstance().showError("This process is read-only, your changes will not be saved");
          this.addAndSelectProcess();

        } else {
          var remote:RemoteObject = ServiceLocator.getInstance().getRemoteObject("fileNavigator");
          var token:AsyncToken = remote.whoTakeToken(process);
          // add fonctions to call on result
          token.addResponder(new Responder(checkWhoGetTokenResult, fault));
        }
        break;
      }
    }
  }

  private function checkWhoGetTokenResult(event:ResultEvent):void {
    var userTakeToken:String = event.result as String;
    if (userTakeToken != DesignerModelLocator.getInstance().userName) {
      processEditorModel.readWrite = false;
      ErrorManager.getInstance().updateTitle("Warning : writing permission denied");
      ErrorManager.getInstance().showError("This process is already open by " + userTakeToken +
          " so it is opened in read only.\nYour changes will not be saved");
    }
    this.addAndSelectProcess();
  }

  private function addAndSelectProcess():void {
    // add process model to model locator
    DesignerModelLocator.getInstance().openedProcesses.addItem(processEditorModel);
    // select process model
    DesignerModelLocator.getInstance().selectedProcessEditor = processEditorModel;

    CursorManager.removeBusyCursor();
    DesignerModelLocator.getInstance().fileNavigatorModel.isEnabledDoubleClick = true;
  }
}
}
