/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

package org.ow2.orchestra.designer.control {
import com.adobe.cairngorm.CairngormError;
import com.adobe.cairngorm.CairngormMessageCodes;
import com.adobe.cairngorm.control.FrontController;

import org.ow2.orchestra.designer.commands.AccessPermissionUpdateCMD;
import org.ow2.orchestra.designer.commands.AddProcessElementCMD;
import org.ow2.orchestra.designer.commands.AddProcessElementWithSequenceFlowCMD;
import org.ow2.orchestra.designer.commands.AdminProcessesFetchCMD;
import org.ow2.orchestra.designer.commands.ChangeAccessSettingsProcessCMD;
import org.ow2.orchestra.designer.commands.CreateSequenceFlowCMD;
import org.ow2.orchestra.designer.commands.DeleteSelectedProcessElementsCMD;
import org.ow2.orchestra.designer.commands.FileNavigatorFetchTreeModelCMD;
import org.ow2.orchestra.designer.commands.FileNavigatorItemDeleteCMD;
import org.ow2.orchestra.designer.commands.GetUsernameCMD;
import org.ow2.orchestra.designer.commands.ListUsersOfDesignerCMD;
import org.ow2.orchestra.designer.commands.ModelDeployCMD;
import org.ow2.orchestra.designer.commands.ModelExportCMD;
import org.ow2.orchestra.designer.commands.ModelSaveCMD;
import org.ow2.orchestra.designer.commands.MoveSelectedElementsCMD;
import org.ow2.orchestra.designer.commands.NewProcessCMD;
import org.ow2.orchestra.designer.commands.NewProjectCMD;
import org.ow2.orchestra.designer.commands.PingServerCMD;
import org.ow2.orchestra.designer.commands.ProcessCloseCMD;
import org.ow2.orchestra.designer.commands.ProcessOpenCMD;
import org.ow2.orchestra.designer.commands.ProcessSelectCMD;
import org.ow2.orchestra.designer.commands.RedoCMD;
import org.ow2.orchestra.designer.commands.SelectProcessElementsCMD;
import org.ow2.orchestra.designer.commands.UndoCMD;
import org.ow2.orchestra.designer.commands.UpdateElementPropertyCMD;
import org.ow2.orchestra.designer.commands.WsdlNavigatorFetchModelCMD;
import org.ow2.orchestra.designer.events.AccessPropertiesEvent;
import org.ow2.orchestra.designer.events.DesignerToolBarAppEvent;
import org.ow2.orchestra.designer.events.ElementPropertyChangeEvent;
import org.ow2.orchestra.designer.events.FileNavigatorEvent;
import org.ow2.orchestra.designer.events.ListUserOfDesignerEvent;
import org.ow2.orchestra.designer.events.ProcessEditorEvent;
import org.ow2.orchestra.designer.events.ProcessEvent;
import org.ow2.orchestra.designer.events.ProcessModificationEvent;
import org.ow2.orchestra.designer.events.ProcessSelectionElementCoordinatesChangeEvent;
import org.ow2.orchestra.designer.events.ProcessSelectionElementsEvent;
import org.ow2.orchestra.designer.events.PingServerEvent;
import org.ow2.orchestra.designer.events.SequenceFlowEvent;
import org.ow2.orchestra.designer.events.UndoRedoEvent;
import org.ow2.orchestra.designer.events.WsdlNavigatorEvent;

public class ProcessController extends FrontController {
  /**
   * Singleton Pattern.
   */
  private static var instance:ProcessController;

  public static function getInstance():ProcessController {
    if (instance == null) {
      new ProcessController();
    }

    return instance;
  }

  public function ProcessController() {
    super();
    if (instance != null) {
      throw new CairngormError(CairngormMessageCodes.SINGLETON_EXCEPTION, "ProcessController");
    }

    //The mxml doesn't use the getInstance method so we have to adapt the singleton
    // pattern by setting the instance here.
    instance = this;

    //Open process file
    addCommand(FileNavigatorEvent.PROCESS_OPEN, ProcessOpenCMD);

    //Close process
    addCommand(ProcessEvent.PROCESS_CLOSE, ProcessCloseCMD);

    //Select process
    addCommand(ProcessEvent.PROCESS_SELECT, ProcessSelectCMD);

    //Get WsdlNavigator content
    addCommand(WsdlNavigatorEvent.FETCH_MODEL, WsdlNavigatorFetchModelCMD);

    //Get FileNavigator content
    addCommand(FileNavigatorEvent.FETCH_MODEL, FileNavigatorFetchTreeModelCMD);

    // Delete a projet or a process
    addCommand(FileNavigatorEvent.ITEM_DELETE, FileNavigatorItemDeleteCMD);

    // Add a new project
    addCommand(FileNavigatorEvent.PROJECT_CREATE, NewProjectCMD);

    // Add a new process
    addCommand(FileNavigatorEvent.PROCESS_CREATE, NewProcessCMD);

    // delete selected process elements
    addCommand(ProcessEvent.PROCESS_SELECTED_ELEMENTS_DELETE, DeleteSelectedProcessElementsCMD);

    // add process element
    addCommand(ProcessModificationEvent.PROCESS_ELEMENT_ADD, AddProcessElementCMD);

    // add process element with sequenceFlow
    addCommand(ProcessModificationEvent.PROCESS_ELEMENT_ADD_WITH_SEQUENCE, AddProcessElementWithSequenceFlowCMD);

    // select process elements
    addCommand(ProcessSelectionElementsEvent.PROCESS_ELEMENTS_SELECT, SelectProcessElementsCMD);

    // change a property on a process element
    addCommand(ElementPropertyChangeEvent.PROPERTY_CHANGED, UpdateElementPropertyCMD);

    // change a property on a process element
    addCommand(ProcessSelectionElementCoordinatesChangeEvent.COORDINATES_CHANGE, MoveSelectedElementsCMD);

    // add a sequenceFlow
    addCommand(SequenceFlowEvent.FLOW_CREATION, CreateSequenceFlowCMD);

    // save a given list of process editor models.
    addCommand(ProcessEditorEvent.SAVE_MODELS, ModelSaveCMD);

    // export a given list of process editor models.
    addCommand(ProcessEditorEvent.EXPORT_MODELS, ModelExportCMD);

    // deploy a given list of process editor models.
    addCommand(ProcessEditorEvent.DEPLOY_MODELS, ModelDeployCMD);

    // undo
    addCommand(UndoRedoEvent.UNDO, UndoCMD);
    // redo
    addCommand(UndoRedoEvent.REDO, RedoCMD);


    //add username
    addCommand(DesignerToolBarAppEvent.ADD_USERNAME, GetUsernameCMD);

    // confirm the modification of access permission
    addCommand(AccessPropertiesEvent.ACCESS_PERMISSIONS_CHANGED, AccessPermissionUpdateCMD);
    // recover the list of users designer
    addCommand(ListUserOfDesignerEvent.LIST_USERS_OF_DESIGNER, ListUsersOfDesignerCMD);
    // change process
    addCommand(AccessPropertiesEvent.PROCESS_CHANGED, ChangeAccessSettingsProcessCMD);

    addCommand(AccessPropertiesEvent.FETCH_ADMIN_PROCESSES, AdminProcessesFetchCMD);

    // refresh token
    addCommand(PingServerEvent.PING_SERVER, PingServerCMD);

  }

}
}
