/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.view.menuBar {
import mx.binding.utils.BindingUtils;
import mx.collections.ArrayCollection;
import mx.controls.MenuBar;
import mx.core.Application;
import mx.events.CloseEvent;
import mx.events.CollectionEvent;
import mx.events.MenuEvent;
import mx.managers.PopUpManager;

import org.act.od.impl.other.Localizator;
import org.act.od.impl.view.AboutWindow;
import org.act.od.impl.view.toolMenuBar.DesignerToolBar;
import org.ow2.orchestra.designer.events.FileNavigatorEvent;
import org.ow2.orchestra.designer.events.ProcessEditorEvent;
import org.ow2.orchestra.designer.events.ProcessEvent;
import org.ow2.orchestra.designer.events.ProcessSelectionElementsEvent;
import org.ow2.orchestra.designer.events.UndoRedoEvent;
import org.ow2.orchestra.designer.model.DesignerModelLocator;
import org.ow2.orchestra.designer.model.ProcessEditorModel;
import org.ow2.orchestra.designer.view.CreateNewFileWindow;
import org.ow2.orchestra.designer.view.OrchestraConnectionSettings;
import org.ow2.orchestra.designer.vo.FileNavigatorOwner;
import org.ow2.orchestra.designer.vo.FileNavigatorProcess;
import org.ow2.orchestra.designer.vo.FileNavigatorProject;

/**
 * Menu Bar for OrDesigner.
 */
public class DesignerMenuBar extends MenuBar {

  private var enabledSave:Boolean;
  private var enabledSaveAll:Boolean;

  /**
   * Initialize DesignerMenuBar.
   */
  public function DesignerMenuBar() {
    //view component
    super();
    enabledSaveAll = true;
    enabledSave = true;
    this.dataProvider = this.getMenu();
    this.percentWidth = 100;

    this.addEventListener(MenuEvent.ITEM_CLICK, menuClick);

    // Active the tab matching to the selected process
    BindingUtils.bindSetter(setSelectedProcessEditor, DesignerModelLocator.getInstance(), "selectedProcessEditor");
    // Active the tab matching to the opened process
    DesignerModelLocator.getInstance().openedProcesses.addEventListener(CollectionEvent.COLLECTION_CHANGE, setOpenedProcesses);

  }


  private function getMenu():ArrayCollection {
    var localizator:Localizator = Localizator.getInstance();
    var menu:ArrayCollection = new ArrayCollection();
    ///////////////
    // file menu //
    ///////////////
    var fileMenu:MenuBarItemWithChildren = new MenuBarItemWithChildren(localizator.getText('menubar.file'));
    // new file menu
    var fileNewMenu:MenuBarItemWithChildren = new MenuBarItemWithChildren(localizator.getText('menubar.file.new'));
    fileMenu.children.addItem(fileNewMenu);
    fileNewMenu.children.addItem(new MenuBarItem(localizator.getText('menubar.file.project'), true, onNewProjectHandle, null, DesignerToolBar.newproject));
    fileNewMenu.children.addItem(new MenuBarItem(localizator.getText('menubar.file.process'), true, onNewProcessHandle, null, DesignerToolBar.newprocess));
    // other entries
    fileMenu.children.addItem(new MenuBarItem(null, true, null, null, null, false, "separator"));
    fileMenu.children.addItem(new MenuBarItem(localizator.getText('menubar.file.close'), true, onCloseHandle));
    fileMenu.children.addItem(new MenuBarItem(localizator.getText('menubar.file.closeall'), true, onCloseAllHandle));
    fileMenu.children.addItem(new MenuBarItem(null, true, null, null, null, false, "separator"));
    if (enabledSave == true) {
      fileMenu.children.addItem(new MenuBarItem(localizator.getText('menubar.file.save'), enabledSave, onSaveHandle, null, DesignerToolBar.save));
    } else {
      fileMenu.children.addItem(new MenuBarItem(localizator.getText('menubar.file.save'), enabledSave, onSaveHandle, null, DesignerToolBar.save_gray));
    }
    if (enabledSaveAll == true) {
      fileMenu.children.addItem(new MenuBarItem(localizator.getText('menubar.file.save'), enabledSaveAll, onSaveHandle, null, DesignerToolBar.saveall));
    } else {
      fileMenu.children.addItem(new MenuBarItem(localizator.getText('menubar.file.save'), enabledSaveAll, onSaveHandle, null, DesignerToolBar.saveall_gray));
    }
    menu.addItem(fileMenu);

    ///////////////
    // Edit menu //
    ///////////////
    var editMenu:MenuBarItemWithChildren = new MenuBarItemWithChildren(localizator.getText('menubar.edit'));
    editMenu.children.addItem(new MenuBarItem(localizator.getText('menubar.edit.cut'), true, onCutHandle, null, DesignerToolBar.cut));
    editMenu.children.addItem(new MenuBarItem(localizator.getText('menubar.edit.copy'), true, onCopyHandle, null, DesignerToolBar.copy));
    editMenu.children.addItem(new MenuBarItem(localizator.getText('menubar.edit.paste'), true, onPasteHandle, null, DesignerToolBar.paste));
    editMenu.children.addItem(new MenuBarItem(null, true, null, null, null, false, "separator"));
    editMenu.children.addItem(new MenuBarItem(localizator.getText('menubar.edit.selectall'), true, onSelectAll));
    editMenu.children.addItem(new MenuBarItem(null, true, null, null, null, false, "separator"));
    editMenu.children.addItem(new MenuBarItem(localizator.getText('menubar.edit.undo'), true, onUndo, null, DesignerToolBar.undo));
    editMenu.children.addItem(new MenuBarItem(localizator.getText('menubar.edit.redo'), true, onRedo, null, DesignerToolBar.redo));
    menu.addItem(editMenu);

    //////////////////
    // Options menu //
    //////////////////
    var optionMenu:MenuBarItemWithChildren = new MenuBarItemWithChildren(localizator.getText('menubar.option'));
    var viewMenu:MenuBarItemWithChildren = new MenuBarItemWithChildren(localizator.getText('menubar.option.view'));
    viewMenu.children.addItem(new MenuBarItem(localizator.getText('menubar.option.view.properties'), true, onDisplayPropertiesHandle, null, null, DesignerModelLocator.getInstance().isPropertiesViewVisible, "check"));
    viewMenu.children.addItem(new MenuBarItem(localizator.getText('menubar.option.view.microimage'), true, onDisplayMicroImageHandle, null, null, DesignerModelLocator.getInstance().isMicroImageViewVisible, "check"));
    optionMenu.children.addItem(new MenuBarItem(localizator.getText('menubar.option.orchestra.connection'), true, onOrchestraSettingsHandle));
    optionMenu.children.addItem(viewMenu);
    menu.addItem(optionMenu);

    ///////////////
    // Help menu //
    ///////////////
    var helpMenu:MenuBarItemWithChildren = new MenuBarItemWithChildren(localizator.getText('menubar.help'));
    helpMenu.children.addItem(new MenuBarItem(localizator.getText('menubar.help.about'), true, onAboutHandle));
    menu.addItem(helpMenu);
    return menu;
  }

  /**
   * Deal with the MenuClick event.
   */
  public function menuClick(event:MenuEvent):void {
    var menuBarItem:MenuBarItem = event.item as MenuBarItem;
    if (menuBarItem != null) {
      if (menuBarItem.handler != null) {
        menuBarItem.handler(event);
      }
    }
  }

  private function onNewProjectHandle(event:MenuEvent):void {
    var newProject:CreateNewFileWindow = CreateNewFileWindow(PopUpManager.createPopUp(Application.application as Application, CreateNewFileWindow, true));
    newProject.setTitle("Project");
    PopUpManager.centerPopUp(newProject);
    newProject.addEventListener(CloseEvent.CLOSE, newProjectResult);
  }

  private function newProjectResult(event:CloseEvent):void {
    var projectName:String = CreateNewFileWindow(event.currentTarget).getText();
    if (projectName != "") {
      var owner:FileNavigatorOwner = new FileNavigatorOwner();
      owner.name = DesignerModelLocator.getInstance().userName;
      var project:FileNavigatorProject = new FileNavigatorProject();
      project.name = projectName;
      project.owner = owner;
      owner.projects.addItem(project);
      new FileNavigatorEvent(FileNavigatorEvent.PROJECT_CREATE, project).dispatch();
    }
  }

  private function onNewProcessHandle(event:MenuEvent):void {
    var newFile:CreateNewFileWindow = CreateNewFileWindow(PopUpManager.createPopUp(Application.application as Application, CreateNewFileWindow, true));
    newFile.setTitle("Process");
    PopUpManager.centerPopUp(newFile);
    newFile.addEventListener(CloseEvent.CLOSE, newFileResult);
  }

  private function newFileResult(event:CloseEvent):void {
    var fileName:String = CreateNewFileWindow(event.currentTarget).getText();
    if (fileName != "") {
      var process:FileNavigatorProcess = new FileNavigatorProcess();
      process.name = fileName;
      process.project.owner.name = DesignerModelLocator.getInstance().userName;
      new FileNavigatorEvent(FileNavigatorEvent.PROCESS_CREATE, process).dispatch();
    }
  }

  private function onCloseHandle(event:MenuEvent):void {
    new ProcessEvent(ProcessEvent.PROCESS_CLOSE, DesignerModelLocator.getInstance().selectedProcessEditor).dispatch();
  }

  private function onCloseAllHandle(event:MenuEvent):void {
    var models:ArrayCollection = new ArrayCollection();
    models.addAll(DesignerModelLocator.getInstance().openedProcesses);
    //iterate over another list
    for each (var model:ProcessEditorModel in models) {
      new ProcessEvent(ProcessEvent.PROCESS_CLOSE, model).dispatch();
    }

  }

  private function onSaveHandle(event:MenuEvent):void {
    var models:ArrayCollection = new ArrayCollection();
    var selectedProcess:ProcessEditorModel = DesignerModelLocator.getInstance().selectedProcessEditor;
    if (selectedProcess != null)
      models.addItem(selectedProcess);
    new ProcessEditorEvent(ProcessEditorEvent.SAVE_MODELS, models).dispatch();
  }

  private function onSaveAllHandle(event:MenuEvent):void {
    var models:ArrayCollection = new ArrayCollection();
    models.addAll(DesignerModelLocator.getInstance().openedProcesses);
    new ProcessEditorEvent(ProcessEditorEvent.SAVE_MODELS, models).dispatch();
  }

  private function onUndo(event:MenuEvent):void {
    new UndoRedoEvent(UndoRedoEvent.UNDO).dispatch();
  }

  private function onRedo(event:MenuEvent):void {
    new UndoRedoEvent(UndoRedoEvent.REDO).dispatch();
  }

  private function onCutHandle(event:MenuEvent):void {
  }

  private function onCopyHandle(event:MenuEvent):void {
  }

  private function onPasteHandle(event:MenuEvent):void {
  }

  private function onSelectAll(event:MenuEvent):void {
    if (DesignerModelLocator.getInstance().selectedProcessEditor != null) {
      var elements:ArrayCollection = new ArrayCollection();
      elements.addAll(DesignerModelLocator.getInstance().selectedProcessEditor.processModel.elements);
      new ProcessSelectionElementsEvent(ProcessSelectionElementsEvent.PROCESS_ELEMENTS_SELECT, DesignerModelLocator.getInstance().selectedProcessEditor, elements).dispatch();
    }

  }

  private function onOrchestraSettingsHandle(event:MenuEvent):void {
    var orchestraSettings:OrchestraConnectionSettings = OrchestraConnectionSettings(PopUpManager.createPopUp(Application
        .application as Application, OrchestraConnectionSettings, true));
    PopUpManager.centerPopUp(orchestraSettings);
  }

  private function onAboutHandle(event:MenuEvent):void {
    var aboutWindow:AboutWindow = AboutWindow(PopUpManager.createPopUp(Application.application as Application, AboutWindow,
        true));
    PopUpManager.centerPopUp(aboutWindow);
  }


  private function onDisplayPropertiesHandle(event:MenuEvent):void {
    var pViewToggled:Boolean = this.dataDescriptor.isToggled(event.item);
    DesignerModelLocator.getInstance().isPropertiesViewVisible = pViewToggled;
  }

  private function onDisplayMicroImageHandle(event:MenuEvent):void {
    var miViewToggled:Boolean = this.dataDescriptor.isToggled(event.item);
    DesignerModelLocator.getInstance().isMicroImageViewVisible = miViewToggled;
  }

  /**
   * deactivate the button "save"
   * @param selectedProcessEditor
   */
  private function setSelectedProcessEditor(selectedProcessEditor:ProcessEditorModel):void {
    if (DesignerModelLocator.getInstance().selectedProcessEditor != null) {
      enabledSave = DesignerModelLocator.getInstance().selectedProcessEditor.readWrite;
    } else {
      enabledSave = true;
    }
    this.dataProvider = this.getMenu();
  }

  /**
   *  deactivate the button "saveAll"
   * @param event
   */
  private function setOpenedProcesses(event:CollectionEvent):void {
    var value:Boolean = true;
    if (DesignerModelLocator.getInstance().openedProcesses.length > 0) {
      value = false;
      for each (var processEditorModel:Object in DesignerModelLocator.getInstance().openedProcesses) {
        if ((processEditorModel as ProcessEditorModel).readWrite == true) {
          value = true;
          break;
        }
      }
    }
    enabledSaveAll = value;
    this.dataProvider = this.getMenu();
  }

}
}
