/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


package org.ow2.orchestra.designer.view.processEditor {
  import flash.display.DisplayObject;
  import flash.events.MouseEvent;
  import flash.geom.Point;

  import mx.collections.ArrayCollection;
  import mx.events.FlexEvent;
  import mx.states.SetEventHandler;
  import mx.states.State;

  import org.ow2.orchestra.designer.bpmn.model.SequenceFlowModel;
  import org.ow2.orchestra.designer.bpmn.views.AbstractElementView;
  import org.ow2.orchestra.designer.bpmn.views.ProcessPoolView;
  import org.ow2.orchestra.designer.bpmn.views.SequenceFlowView;

  public class ProcessEditorDefaultStateHelper extends ProcessEditorFlowStateHelper {

    public function ProcessEditorDefaultStateHelper(processEditorView:ProcessEditorView) {
      super(processEditorView);
    }


    /**
     * Generate default state.
     * In the default state:
     * - the selection rectangle is hidden
     * - on mouse down on the processView canvas, this view goes to the "selecting" state.
     */
    public function addDefaultState():void {
      var state:State = new State();
      state.name = "default";
      processEditorView.states.push(state);

      // handler for mouse down event
      var mouseDownListener:SetEventHandler = new SetEventHandler(processEditorView, MouseEvent.MOUSE_DOWN);
      mouseDownListener.handlerFunction = defaultStateMouseDown;
      state.overrides.push(mouseDownListener);

      state.addEventListener(FlexEvent.ENTER_STATE, defaultStateEnter);
    }

    /**
     * Show context panel when entering default state if only one element is selected.
     * @param event
     */
    private function defaultStateEnter(event:FlexEvent):void {
      processEditorView.showElementContextPanel();
    }


    /**
     * handler for mouse down event when this view is in the "default" state.
     * On mouse down on a element or on the canvas, this view goes to the "selecting" state.
     * On mouse down on a selected element, this view goes to the "moving" state.
     */
    private function defaultStateMouseDown(event:MouseEvent):void {


      // Get mouse event position. The coordinates are relative to the target element
      var targetLocalMousePoint:Point = new Point(event.localX, event.localY);
      // Transform the coordinates to be relative to this view
      var mousePoint:Point = processEditorView.globalToLocal(DisplayObject(event.target).localToGlobal(targetLocalMousePoint));

      // set the selectionRect position
      processEditorView.selectionRect.x = mousePoint.x;
      processEditorView.selectionRect.y = mousePoint.y;
      // reset the selectionRect size
      processEditorView.selectionRect.width = 1;
      processEditorView.selectionRect.height = 1;

      if (processEditorView.processEditorModel.selectedElements.length == 1 && processEditorView.processEditorModel.selectedElements.getItemAt(0) is SequenceFlowModel) {
        var seqFlowView:SequenceFlowView = processEditorView.processView.getElementViewByModel(processEditorView.processEditorModel
                                                                                                       .selectedElements.getItemAt(0) as SequenceFlowModel) as SequenceFlowView;
        for each (var displayObject:DisplayObject in seqFlowView.getChildren()) {
          if (displayObject is SequenceFlowSelectionAnchor) {
            if (displayObject.getRect(processEditorView).containsPoint(mousePoint)) {
              if (SequenceFlowSelectionAnchor(displayObject).isSource) {
                processEditorView.currentState = "movingSourceFlow";
              } else {
                processEditorView.currentState = "movingTargetFlow";
              }
              break;
            }
          }
        }
      }
      if (processEditorView.currentState == "default") {
        // Ignore event if the mouse in over the context panel
        // Mouse is over the context panel if
        // - the context panel is in default state and the context panel rect contains the mouse point and the element rect does not contain the mouse point
        // - the context panel is in another state (e.g. remove state balloon) and the context panel rect contains the mouse point
        if (!processEditorView.elementContextPanel.visible
                    || !processEditorView.elementContextPanel.getRect(processEditorView).containsPoint(mousePoint)
                || (processEditorView.elementContextPanel.currentState == "default"
                && processEditorView.elementContextPanel.attachedView.containsPoint(mousePoint, processEditorView)
                && !(processEditorView.elementContextPanel.attachedView is SequenceFlowView))) {
          // check if mouse is on a element view or on the canvas.
          // find list of elements that are contained in the selection area
          var listSelectedElements:ArrayCollection = new ArrayCollection();
          for each (var object:DisplayObject in processEditorView.processView.getChildren()) {
            if (!(object is ProcessPoolView) && object is AbstractElementView) {
              if (AbstractElementView(object).containsPoint(mousePoint, processEditorView)) {
                // the process element view is contained in the selection area,
                // the attached model should be selected.
                listSelectedElements.addItem((object as AbstractElementView).model);
              }
            }
          }
          if (listSelectedElements.length != 0
                  && processEditorView.processEditorModel.selectedElements.contains(listSelectedElements.getItemAt(0))) {
            // if mouse is on a selected element go to "moving" state
            // go to move state
            processEditorView.currentState = "moving";
          } else {
            // go to "selecting" state
            processEditorView.currentState = "selecting";
          }
        }
      }
    }
  }
}
