/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.view.processEditor {
  import flash.display.DisplayObject;
  import flash.events.MouseEvent;
  import flash.geom.Point;

  import mx.collections.ArrayCollection;
  import mx.events.FlexEvent;
  import mx.states.SetEventHandler;
  import mx.states.State;
  import mx.styles.StyleManager;

  import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;
  import org.ow2.orchestra.designer.bpmn.model.SequenceFlowModel;
  import org.ow2.orchestra.designer.bpmn.views.SequenceFlowView;
  import org.ow2.orchestra.designer.events.ElementPropertyChangeEvent;

  public class ProcessEditorMovingFlowStateHelper extends ProcessEditorFlowStateHelper {


    private static var myCursorCopy:Class = StyleManager.getStyleDeclaration('DragManager').getStyle('copyCursor');
    private static var myCursorRejected:Class = StyleManager.getStyleDeclaration('DragManager').getStyle('rejectCursor');

    public function ProcessEditorMovingFlowStateHelper(processEditorView:ProcessEditorView) {
      super(processEditorView);
    }


    public function addMovingSourceState():void {
      var state:State = new State();
      state.name = "movingSourceFlow";
      processEditorView.states.push(state);

      var mouseMoveListener:SetEventHandler = new SetEventHandler(processEditorView, MouseEvent.MOUSE_MOVE);
      mouseMoveListener.handlerFunction = this.handleMouseMoveSource;
      state.overrides.push(mouseMoveListener);

      var mouseClickListener:SetEventHandler = new SetEventHandler(processEditorView, MouseEvent.MOUSE_UP);
      mouseClickListener.handlerFunction = this.handleMouseUpSource;
      state.overrides.push(mouseClickListener);
      state.addEventListener(FlexEvent.EXIT_STATE, movingStateExit);
    }

    public function addMovingTargetState():void {
      var state:State = new State();
      state.name = "movingTargetFlow";
      processEditorView.states.push(state);

      var mouseMoveListener:SetEventHandler = new SetEventHandler(processEditorView, MouseEvent.MOUSE_MOVE);
      mouseMoveListener.handlerFunction = this.handleMouseMoveTarget;
      state.overrides.push(mouseMoveListener);

      var mouseClickListener:SetEventHandler = new SetEventHandler(processEditorView, MouseEvent.MOUSE_UP);
      mouseClickListener.handlerFunction = this.handleMouseUpTarget;
      state.overrides.push(mouseClickListener);
      state.addEventListener(FlexEvent.EXIT_STATE, movingStateExit);
    }

    private function movingStateExit(event:FlexEvent):void {
      var sequenceFlowView:SequenceFlowView = processEditorView.processView.getElementViewByModel(processEditorView.processEditorModel
              .selectedElements.getItemAt(0) as SequenceFlowModel) as SequenceFlowView;
      // when exiting moving states Redraw sequence flow
      //This handles mouse out events and click on non-abstract element with position
      sequenceFlowView.redraw();

      resetCursor();
    }


    private function handleMouseMoveSource(mouseEvent:MouseEvent):void {
      this.handleMouseMove(mouseEvent, true);
    }

    private function handleMouseUpSource(mouseEvent:MouseEvent):void {
      this.handleMouseUp(mouseEvent, true);
    }

    private function handleMouseMoveTarget(mouseEvent:MouseEvent):void {
      this.handleMouseMove(mouseEvent, false);
    }

    private function handleMouseUpTarget(mouseEvent:MouseEvent):void {
      this.handleMouseUp(mouseEvent, false);
    }

    protected function handleMouseMove(mouseEvent:MouseEvent, isSource:Boolean):void {
      processEditorView.hideElementContextPanel();

      // Get mouse event position. The coordinates are relative to the target element
      var targetLocalMousePoint:Point = new Point(mouseEvent.localX, mouseEvent.localY);
      // Transform the coordinates to be relative to this view
      var mousePoint:Point = processEditorView.globalToLocal(DisplayObject(mouseEvent
              .target).localToGlobal
              (targetLocalMousePoint));
      // A sequence flow view should be selected
      var sequenceFlowView:SequenceFlowView = processEditorView.processView.getElementViewByModel(processEditorView.processEditorModel
              .selectedElements.getItemAt(0) as SequenceFlowModel) as SequenceFlowView;
      if (isSource) {
        sequenceFlowView.sourcePoint = mousePoint;
        this.updateDropMouseCursor(this.allowFlowSource(mouseEvent));
      } else {
        sequenceFlowView.targetPoint = mousePoint;
        this.updateDropMouseCursor(this.allowFlowTarget(mouseEvent));
      }
    }

    private function handleMouseUp(mouseEvent:MouseEvent, isSource:Boolean):void {
      // Check if object under mouse is a valid target
      if ((isSource && allowFlowSource(mouseEvent))
              || (!isSource && allowFlowTarget(mouseEvent))) {
        var listSelectedElements:ArrayCollection = new ArrayCollection();
        listSelectedElements = getElementUnderMouse(mouseEvent);
        var sequenceFlowView:SequenceFlowView = processEditorView.processView.getElementViewByModel(processEditorView.processEditorModel
                .selectedElements.getItemAt(0) as SequenceFlowModel) as SequenceFlowView;

        if (listSelectedElements.length == 1) {
          //Dispatch creating flow event in order to create flow from source element to target element
          var targetElement:AbstractElementWithPosition =
                  listSelectedElements.getItemAt(0) as AbstractElementWithPosition;
          if (isSource) {
            new ElementPropertyChangeEvent(ElementPropertyChangeEvent.PROPERTY_CHANGED, sequenceFlowView.model, "sourceElement",
                    targetElement).dispatch();
          } else {
            new ElementPropertyChangeEvent(ElementPropertyChangeEvent.PROPERTY_CHANGED, sequenceFlowView.model, "targetElement",
                    targetElement).dispatch();
          }
        }
      }
      processEditorView.currentState = "default";

    }
  }

}
