package org.ow2.orchestra.designer.view.properties {
  import mx.binding.utils.BindingUtils;
  import mx.collections.ArrayCollection;
  import mx.containers.Box;
  import mx.events.CollectionEvent;
  import mx.events.DataGridEvent;

  import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
  import org.ow2.orchestra.designer.bpmn.model.EventModel;
  import org.ow2.orchestra.designer.bpmn.model.EventType;
  import org.ow2.orchestra.designer.bpmn.model.GatewayModel;
  import org.ow2.orchestra.designer.bpmn.model.GatewayType;
  import org.ow2.orchestra.designer.bpmn.model.PoolModel;
  import org.ow2.orchestra.designer.bpmn.model.SequenceFlowModel;
  import org.ow2.orchestra.designer.bpmn.model.TaskModel;
  import org.ow2.orchestra.designer.bpmn.model.TaskType;
  import org.ow2.orchestra.designer.model.DesignerModelLocator;
  import org.ow2.orchestra.designer.model.ProcessEditorModel;

  /**
   * The view for figure's attribute.
   */
  public class PropertyView extends Box {
    /**
     * The source for "properties" image data binding.
     */
    [Bindable]
    [Embed(source="/../assets/icon/container/properties.gif")]
    public var properties:Class;

    private var propertiesDataGrid:PropertyDataGrid = new PropertyDataGrid();
    private var processEditorModel:ProcessEditorModel;

    /**
     * Initialize figure's attribute view.
     */
    public function PropertyView() {
      super();

      propertiesDataGrid.percentHeight = 100;
      propertiesDataGrid.percentWidth = 100;

      this.addChild(propertiesDataGrid);
      this.percentHeight = 100;
      this.percentWidth = 100;
      this.label = "Properties";
      this.icon = properties;

      // Active the tab matching to the selected process
      BindingUtils.bindSetter(setSelectedProcessEditor, DesignerModelLocator.getInstance(), "selectedProcessEditor");

      // set empty properties view
      selectedElementChanged();

      propertiesDataGrid.addEventListener(DataGridEvent.ITEM_EDIT_BEGINNING, checkEditableField, false, 0, true);
    }

    /**
     * Active the tab matching to the selected process
     */
    private function setSelectedProcessEditor(selectedProcessEditor:ProcessEditorModel):void {
      if (processEditorModel != null) {
        processEditorModel.selectedElements.removeEventListener(CollectionEvent.COLLECTION_CHANGE, selectedElementChanged);
      }
      processEditorModel = selectedProcessEditor;
      if (selectedProcessEditor != null) {
        selectedProcessEditor.selectedElements.addEventListener(CollectionEvent.COLLECTION_CHANGE, selectedElementChanged);
      }
      selectedElementChanged();
    }

    /**
     * When the collection is modified then we update the data grid.
     * Note: when a element of the collection is updated (one of its values changed) the a collection event
     * {@link mx.events.CollectionEventKind@UPDATE} is dispatched. So we do not have to listen on properties changed events for
     * selected
     * elements.
     * @param event A collection event
     */
    private function selectedElementChanged(event:CollectionEvent = null):void {
      var propertiesData:ArrayCollection = new ArrayCollection();
      if (processEditorModel != null) {
        if (processEditorModel.selectedElements.length == 1) {
          var selectedElement:AbstractElement = processEditorModel.selectedElements.getItemAt(0) as AbstractElement;
          if (selectedElement is EventModel) {
            // Properties for an event
            var selectedEvent:EventModel = selectedElement as EventModel;
            propertiesData.addItem(new TextProperty("id", "Id", selectedEvent.id));
            propertiesData.addItem(new TextProperty("name", "Name", selectedEvent.name, true));
            propertiesData.addItem(new EnumProperty("eventType", "Type", selectedEvent.eventType, EventType.constants));
            if (selectedEvent.eventType == EventType.RECEIVE_MESSAGE) {
              propertiesData.addItem(new TextProperty("variableName", "Variable", selectedEvent.variableName, true));
              if (selectedEvent.operation != null) {
                propertiesData.addItem(new TextProperty("operationName", "Operation", selectedEvent.operation.name, true));
                propertiesData.addItem(new TextProperty("interfaceName", "Interface (PortType)", selectedEvent.operation.interfaceModel.name, true));
              } else {
                propertiesData.addItem(new TextProperty("operationName", "Operation", "", true));
                propertiesData.addItem(new TextProperty("interfaceName", "Interface (PortType)", "", true));
              }
            } else if (selectedEvent.eventType == EventType.SEND_MESSAGE) {
              propertiesData.addItem(new OperationProperty("operation", "Operation (Interface)", selectedEvent.operation));
              propertiesData.addItem(new TextProperty("expression", "Input XPath Expression", selectedEvent.expression, true));
            } else if (selectedEvent.eventType == EventType.TIMER) {
              propertiesData.addItem(new TextProperty("timerDate", "Timer date", selectedEvent.timerDate, true));
              propertiesData.addItem(new TextProperty("timerDuration", "Timer duration", selectedEvent.timerDuration, true));
            }
          } else if (selectedElement is GatewayModel) {
            // Properties for a gateway
            var selectedGateway:GatewayModel = selectedElement as GatewayModel;
            propertiesData.addItem(new TextProperty("id", "Id", selectedGateway.id));
            propertiesData.addItem(new TextProperty("name", "Name", selectedGateway.name, true));
            propertiesData.addItem(new EnumProperty("gatewayType", "Type", selectedGateway.gatewayType, GatewayType.constants));
          } else if (selectedElement is SequenceFlowModel) {
            // Properties for a sequenceFlow
            var selectedSequenceFlow:SequenceFlowModel = selectedElement as SequenceFlowModel;
            propertiesData.addItem(new TextProperty("id", "Id", selectedSequenceFlow.id));
            propertiesData.addItem(new TextProperty("name", "Name", selectedSequenceFlow.name, true));
            if (selectedSequenceFlow.sourceElement is GatewayModel
                    && (selectedSequenceFlow.sourceElement as GatewayModel).gatewayType == GatewayType.EXCLUSIVE) {
              propertiesData.addItem(new TextProperty("condition", "Condition", selectedSequenceFlow.condition, true));
            }

          } else if (selectedElement is TaskModel) {
            // Properties for a task
            var selectedTask:TaskModel = selectedElement as TaskModel;
            propertiesData.addItem(new TextProperty("id", "Id", selectedTask.id));
            propertiesData.addItem(new TextProperty("name", "Name", selectedTask.name, true));
            propertiesData.addItem(new EnumProperty("taskType", "Type", selectedTask.taskType, TaskType.constants));
            if (selectedTask.taskType == TaskType.SERVICE) {
              propertiesData.addItem(new OperationProperty("operation", "Operation (Interface)", selectedTask.operation, true));
              if (selectedTask.operation != null) {
                propertiesData.addItem(new TextProperty("inputExpression", "Input XPath Expression", selectedTask.inputExpression, true));
                if (selectedTask.operation.outMessageType != null) {
                  propertiesData.addItem(new TextProperty("outputVariableName", "Output Variable", selectedTask.outputVariableName, true));
                }
              }
            } else if (selectedTask.taskType == TaskType.USER) {
              propertiesData.addItem(new TextProperty("inputExpression", "Input XPath Expression", selectedTask.inputExpression, true));
              propertiesData.addItem(new TextProperty("outputVariableName", "Output Variable", selectedTask.outputVariableName, true));
              propertiesData.addItem(new TextProperty("potentialOwnerExpression", "Potential Owner Xpath Expression", selectedTask.potentialOwnerExpression, true));
            }

          } else if (selectedElement is PoolModel) {
            // Properties for a pool
            var selectedPool:PoolModel = selectedElement as PoolModel;
            propertiesData.addItem(new TextProperty("id", "Id", selectedPool.id));
            propertiesData.addItem(new TextProperty("name", "Name", selectedPool.name));
          }
        } else if (processEditorModel.selectedElements.length == 0) {
          // properties for the process
          propertiesData.addItem(new TextProperty("id", "Id", processEditorModel.processModel.id));
          propertiesData.addItem(new TextProperty("name", "Name", processEditorModel.processModel.name));
        }
      }
      propertiesDataGrid.dataProvider = propertiesData;
    }

    private function checkEditableField(event:DataGridEvent):void {
      var index:int = propertiesDataGrid.indicesToIndex(event.rowIndex, event.columnIndex);
      var propertiesData:ArrayCollection = ArrayCollection(propertiesDataGrid.dataProvider);
      var property:IProperty = propertiesData.getItemAt(index) as IProperty;
      if (property.itemEditor == null) {
        event.preventDefault();
      }
    }
  }
}
